// -*- C++ -*-
// $RCSfile: blockpre_double.h,v $
// $Revision: 1.5 $
// $Author: langer $
// $Date: 2005-10-11 19:55:41 $

/* This software was produced by NIST, an agency of the U.S. government,
 * and by statute is not subject to copyright in the United States.
 * Recipients of this software assume all responsibilities associated
 * with its operation, modification and maintenance. However, to
 * facilitate maintenance we ask that before distributing modified
 * versions of this software, you first contact the authors at
 * oof_manager@ctcms.nist.gov. 
 */

#include <oofconfig.h>

#ifndef BLOCKPRE_H
#define BLOCKPRE_H

#include "SparseLib++/vecdefs.h"
#include "SparseLib++/mvv.h"
#include "engine/SparseLib++/comprow_double.h"
#include "engine/smallmatrix.h"

#include <map>

class UnitPreconditioner;
class PreconditionerCore;

// The "Block" class represents the actual blocks inside the
// preconditioners.  They know the local block matrix, in dense form.
// The relationship between local block rows and cols and main-matrix
// rows and cols is encoded in the "rows" and "cols" vector, each
// element of which is the row/col in the main matrix to which that
// position corresponds, i.e. element (i,j) of the block corresponds
// to element (rows[i], cols[j]) of the main matrix.

class Block {
private:
  int id_;
  std::vector<int> rows, cols;
  SmallMatrix *mtx, *rhs;
public:
  Block(int);
  ~Block();

  int id() const { return id_; }
  int nrows() const { return rows.size(); }
  int ncols() const { return cols.size(); }

  void new_row(int);
  void new_col(int);
  
  void assign(const CompRow_Mat_double &);
  void assign_rhs(const VECTOR_double &);

  void solve(VECTOR_double &);
  void trans_solve(VECTOR_double &);
};


typedef std::map<int, Block> Blockmap;

// The block preconditioner takes a list of row and column group
// identities.  Zero and positive integers define "regular blocks",
// and negative integers define a single "special block".  The special
// block is presumed to be large, and is preconditioned separately.
// The regular blocks are presumed to be small enough to be
// efficiently directly solved.  The preconditioner M consists of the
// preconditioned special block, and the sum of the regular blocks.
// Matrix elements which are outside of any block are excluded from M.

class CompRow_BlockPreconditioner_double {
private:
  VECTOR_double diags;
  PreconditionerCore *subconditioner;
  const std::vector<int> &rowgroups, &colgroups;
  mutable Blockmap blocks; // Blocks get modified during "solve".
  // These are maps used to extract the in-plane submatrix.
  std::vector<int> in_plane_rows, in_plane_cols;
  CompRow_Mat_double A_inplane;
  
 public:
  CompRow_BlockPreconditioner_double(const CompRow_Mat_double &A,
				     const std::vector<int> &rowgroups,
				     const std::vector<int> &colgroups,
				     UnitPreconditioner *inplane);
  ~CompRow_BlockPreconditioner_double(void);

  VECTOR_double solve(const VECTOR_double &x) const;
  VECTOR_double trans_solve(const VECTOR_double &x) const;
};



#endif // BLOCKPRE_H
