// -*- C++ -*-
// $RCSfile: forcedensity.C,v $
// $Revision: 1.9 $
// $Author: langer $
// $Date: 2007-04-27 15:58:53 $

/* This software was produced by NIST, an agency of the U.S. government,
 * and by statute is not subject to copyright in the United States.
 * Recipients of this software assume all responsibilities associated
 * with its operation, modification and maintenance. However, to
 * facilitate maintenance we ask that before distributing modified
 * versions of this software, you first contact the authors at
 * oof_manager@ctcms.nist.gov. 
 */

#include <oofconfig.h>
#include "common/coord.h"
#include "forcedensity.h"
#include "engine/csubproblem.h"
#include "engine/element.h"
#include "engine/elementnodeiterator.h"
#include "engine/field.h"
#include "engine/flux.h"
#include "engine/indextypes.h"
#include "engine/material.h"
#include "engine/femesh.h"
#include "common/space.h"
#include "common/trace.h"
#include "common/ooferror.h"

ForceDensity::ForceDensity(PyObject *reg, const std::string &nm,
			   double x, double y)
  : Property(nm,reg),
    gx(x),
    gy(y)
{
  displacement = dynamic_cast<TwoVectorField*>(Field::getField("Displacement"));
  stress_flux = dynamic_cast<SymmetricTensorFlux*>(Flux::getFlux("Stress"));
}

void ForceDensity::precompute(FEMesh*) {
}

int ForceDensity::integration_order(const CSubProblem*, const Element *el) const
{
  return 0;
}

void ForceDensity::fluxrhs(const FEMesh *mesh, const Element *element,
			   const Flux *flux, FluxData *fluxdata,
			   const MasterPosition &x) const 
{
  
  // For mass density, only need to set the appropriate vector,
  // it's the same at every point. 
  
  if(*flux != *stress_flux) {
    throw ErrProgrammingError("Unexpected flux", __FILE__, __LINE__);
  }
  
  fluxdata->rhs_element(0) -= gx;
  fluxdata->rhs_element(1) -= gy;
}

