/* -*- C++ -*-
 $RCSfile: diffusionRHS.C,v $
 $Revision: 1.2 $
 $Author: olgak $
 $Date: 2007-07-20 15:37:18 $

 This software was produced by NIST, an agency of the U.S. government,
 and by statute is not subject to copyright in the United States.
 Recipients of this software assume all responsibilities associated
 with its operation, modification and maintenance. However, to
 facilitate maintenance we ask that before distributing modified
 versions of this software, you first contact the authors at
 oof_manager@ctcms.nist.gov. */ 
 
#include "image/SEGMENTATION/diffusionRHS.h"
#include "image/SEGMENTATION/mask.h"

DiffusionRHS::DiffusionRHS()  :gray(ICoord(1,1),0.0)
{
	time = 0;
	size = ICoord(1,1);
}

AntiGeometric::AntiGeometric(double t) {
  	gray = DoubleArray(ICoord(1,1));
	time = t;
	size = ICoord(1,1);
}

DoubleArray AntiGeometric::timestep(DoubleArray g){
	gray = DoubleArray(g);
	size = gray.size();
	ICoord center;
	DoubleArray newgray = DoubleArray(size);
	Mask dxmask = Mask(Mask::X_DERIVATIVE_MASK);
	Mask dymask = Mask(Mask::Y_DERIVATIVE_MASK);
	Mask dxxmask = Mask(Mask::XX_DERIVATIVE_MASK);
	Mask dyymask = Mask(Mask::YY_DERIVATIVE_MASK);
	Mask dxymask = Mask(Mask::XY_DERIVATIVE_MASK);
	DoubleArray dx_mask = dxmask.applyMask(gray);
	DoubleArray dy_mask = dymask.applyMask(gray);
	DoubleArray dxx_mask = dxxmask.applyMask(gray);
	DoubleArray dyy_mask = dyymask.applyMask(gray);
	DoubleArray dxy_mask = dxymask.applyMask(gray);
	for (DoubleArray::iterator i = gray.begin(); i !=gray.end(); ++i){
		center = i.coord();
		double change;
		double a = dx_mask[center]*dx_mask[center]*dxx_mask[center];
		double b = dx_mask[center]*dy_mask[center]*dxy_mask[center];
		double c = dy_mask[center]*dy_mask[center]*dyy_mask[center];
		double d = (dx_mask[center]*dx_mask[center]) + (dy_mask[center]*dy_mask[center]);
		if (d == 0)
			change = 0;
		else 
			change = (a +2*b+c)/(d);
		newgray[center] = gray[center] + time*change;
	}
	return newgray;
}

DoubleArray AntiGeometric::change(DoubleArray g){
	DoubleArray changed = timestep(g);
	DoubleArray final = DoubleArray(g.size());
	for (DoubleArray::iterator i = g.begin(); i !=g.end(); ++i){
		ICoord c = i.coord();
		final[c] = changed[c] - g[c];
	}
	return final;
}

Geometric::Geometric(double t) {
  	gray = DoubleArray(ICoord(1,1));
	time = t;
	size = ICoord(1,1);
}

DoubleArray Geometric::timestep(DoubleArray g){
	gray = DoubleArray(g);
	size = gray.size();
	ICoord center;
	DoubleArray newgray = DoubleArray(size);
	Mask dxmask = Mask(Mask::X_DERIVATIVE_MASK);
	Mask dymask = Mask(Mask::Y_DERIVATIVE_MASK);
	Mask dxxmask = Mask(Mask::XX_DERIVATIVE_MASK);
	Mask dyymask = Mask(Mask::YY_DERIVATIVE_MASK);
	Mask dxymask = Mask(Mask::XY_DERIVATIVE_MASK);
	DoubleArray dx_mask = dxmask.applyMask(gray);
	DoubleArray dy_mask = dymask.applyMask(gray);
	DoubleArray dxx_mask = dxxmask.applyMask(gray);
	DoubleArray dyy_mask = dyymask.applyMask(gray);
	DoubleArray dxy_mask = dxymask.applyMask(gray);
	for (DoubleArray::iterator i = gray.begin(); i !=gray.end(); ++i){
		center = i.coord();
		double change;
		double a = dx_mask[center]*dx_mask[center]*dyy_mask[center];
		double b = dx_mask[center]*dy_mask[center]*dxy_mask[center];
		double c = dy_mask[center]*dy_mask[center]*dxx_mask[center];
		double d = dx_mask[center]*dx_mask[center] + dy_mask[center]*dy_mask[center];
		if (d == 0)
			change = 0;
		else 
			change = (a -2*b+c)/(d);
		newgray[center] = gray[center] + time*change;
	}
	return newgray;
}

DoubleArray Geometric::change(DoubleArray g){
	DoubleArray changed = timestep(g);
	DoubleArray final = DoubleArray(g.size());
	for (DoubleArray::iterator i = g.begin(); i !=g.end(); ++i){
		ICoord c = i.coord();
		final[c] = changed[c] - g[c];
	}
	return final;
}


LargeLaplace::LargeLaplace(double t) {
  	gray = DoubleArray(ICoord(1,1));
	time = t;
	size = ICoord(1,1);
}

DoubleArray LargeLaplace::timestep(DoubleArray gray){
	DoubleArray changes = DoubleArray(gray.size());
	DoubleArray newgray = DoubleArray(gray.size());
	ICoord center;
	Mask mask = Mask(Mask::LAPLACE_MASK);
	changes = mask.applyMask(gray);
	for (DoubleArray::iterator i = gray.begin();i !=gray.end(); ++i){
		center = i.coord();
		newgray[center] = gray[center] + time*changes[center];
	}
	return newgray;
}

DoubleArray LargeLaplace::change(DoubleArray g){
	DoubleArray changed = timestep(g);
	DoubleArray final = DoubleArray(g.size());
	for (DoubleArray::iterator i = g.begin(); i !=g.end(); ++i){
		ICoord c = i.coord();
		final[c] = changed[c] - g[c];
	}
	return final;
}

Laplace::Laplace(double t) {
  	gray = DoubleArray(ICoord(1,1));
	time = t;
	size = ICoord(1,1);
}

DoubleArray Laplace::timestep(DoubleArray gray){
	DoubleArray changes = DoubleArray(gray.size());
	DoubleArray newgray = DoubleArray(gray.size());
	ICoord center;
	Mask mask = Mask(Mask::SMALL_LAPLACE_MASK);
	changes = mask.applyMask(gray);
	for (DoubleArray::iterator i = gray.begin();i !=gray.end(); ++i){
		center = i.coord();
		newgray[center] = gray[center] + time*changes[center];
	}
	return newgray;
}

DoubleArray Laplace::change(DoubleArray g){
	DoubleArray changed = timestep(g);
	DoubleArray final = DoubleArray(g.size());
	for (DoubleArray::iterator i = g.begin(); i !=g.end(); ++i){
		ICoord c = i.coord();
		final[c] = changed[c] - g[c];
	}
	return final;
}

LargeGaussian::LargeGaussian() {
  	gray = DoubleArray(ICoord(1,1));
	size = ICoord(1,1);
}

DoubleArray LargeGaussian::timestep(DoubleArray gray){
	DoubleArray changes = DoubleArray(gray.size());
	Mask mask = Mask(Mask::GAUSSIAN_MASK);
	changes = mask.applyMask(gray);
	return changes;
}

DoubleArray LargeGaussian::change(DoubleArray g){
	DoubleArray changed = timestep(g);
	DoubleArray final = DoubleArray(g.size());
	for (DoubleArray::iterator i = g.begin(); i !=g.end(); ++i){
		ICoord c = i.coord();
		final[c] = changed[c] - g[c];
	}
	return final;
}

Gaussian::Gaussian() {
  	gray = DoubleArray(ICoord(1,1));
	size = ICoord(1,1);
}

DoubleArray Gaussian::timestep(DoubleArray gray){
	DoubleArray changes = DoubleArray(gray.size());
	Mask mask = Mask(Mask::SMALL_GAUSSIAN_MASK);
	changes = mask.applyMask(gray);
	return changes;
}

DoubleArray Gaussian::change(DoubleArray g){
	DoubleArray changed = timestep(g);
	DoubleArray final = DoubleArray(g.size());
	for (DoubleArray::iterator i = g.begin(); i !=g.end(); ++i){
		ICoord c = i.coord();
		final[c] = changed[c] - g[c];
	}
	return final;
}
