 
/* -*- C++ -*-
 $RCSfile: fixborders.C,v $
 $Revision: 1.2 $
 $Author: olgak $
 $Date: 2007-07-20 14:17:56 $

 This software was produced by NIST, an agency of the U.S. government,
 and by statute is not subject to copyright in the United States.
 Recipients of this software assume all responsibilities associated
 with its operation, modification and maintenance. However, to
 facilitate maintenance we ask that before distributing modified
 versions of this software, you first contact the authors at
 oof_manager@ctcms.nist.gov. */ 
 
  
#include "image/SEGMENTATION/fixborders.h"

FixBorders::FixBorders(int n, bool c){
	numClump = n;
	connectNeighbors = c;
}


FixBorders::FixBorders(){
	numClump = 0;
	connectNeighbors = 0;
}

DoubleArray FixBorders::fix(DoubleArray original){
	DoubleArray newgray = DoubleArray(original.size());
	newgray = deleteSmallClumps(original, numClump);
	if (connectNeighbors){
		newgray = connectBorders(newgray);
		touchBorders(newgray, numClump);
	}
	return newgray;
}

NoFix::NoFix(){

}

/*
There have been some issues with shading pixels one or two away from the border. THis shades them if the pixel one or two away is checked. 
*/
void FixBorders::touchBorders(DoubleArray &final, double lowerThreshold){
	ICoord size = final.size();
	ICoord r;
	ICoord l;
	ICoord ra;
	ICoord la;
	for (int i = 0; i < size(0); ++i){
		r = ICoord(i, 0);
		l = ICoord(i, size(1)-1);
		ra = ICoord(i,1);
		la = ICoord(i, size(1) -2);
				if (final[ra] == 0 && final[ICoord(i, 2)] == 1)
			final[ra] = 1;
		if (final[la] == 0 && final[ICoord(i, size(1)-3)] == 1)
			final[la] = 1;
		if (final[r] == 0 && final[ICoord(i, 1)] == 1)
			final[r] = 1;
		if (final[l] == 0 && final[ICoord(i, size(1)-2)] == 1)
			final[l] = 1;
	}
	
	for (int i = 0; i < size(1); ++i){
		r = ICoord(0, i);
		l = ICoord(size(0)-1, i);
		ra = ICoord(1, i);
		la = ICoord(size(0)-2, i);
		if (final[ra] == 0 && final[ICoord(2, i)] == 1)
			final[ra] = 1;
		if (final[la] == 0 && final[ICoord(size(0)-3, i)] == 1)
			final[la] = 1;
		if (final[r] == 0 && final[ICoord(1, i)] == 1)
			final[r] = 1;
		if (final[l] == 0 && final[ICoord(size(0)-2, i)] == 1)
			final[l] = 1;
	}
}

/*
Clears any pixels that have less than 6 neighbors (out of 9)
*/
DoubleArray FixBorders::cleanUpSpeckles(DoubleArray gray){
	ICoord center;
	ICoord size = gray.size();
	for (DoubleArray::iterator i = gray.begin();i !=gray.end(); ++i){
		center = i.coord();
		int num = numSideEmpty(center, gray, 0, 1);
		if ( num > 5){
			gray[center] = 0;
		}
	}
	return gray;

}

/*
Returns number of sides of that pixel that are not checked. 
*/
int FixBorders::numSideEmpty(ICoord center, DoubleArray list, int type, int allSides){
	ICoord size = list.size();
	int num = 0;
	ICoord curr;
	curr = center+ICoord(1,0);
	if (pixelInBounds(curr, size) == true){
		if (list[curr] == type)
			++num;
	}
	curr = center+ICoord(0,1);
	if (pixelInBounds(curr, size) == true){
		if (list[curr] == type)
			++num;
	}
	curr = center+ICoord(-1,0);
	if (pixelInBounds(curr, size) == true){
		if (list[curr] == type)
			++num;
	}
	curr = center+ICoord(0,-1);
	if (pixelInBounds(curr, size) == true){
		if (list[curr] == type)
			++num;
	}
	if (allSides == 1){
		curr = center+ICoord(1,1);
		if (pixelInBounds(curr, size) == true){
			if (list[curr] == type)
				++num;
		}
		
		curr = center+ICoord(-1,-1);
		if (pixelInBounds(curr, size) == true){
			if (list[curr] == type)
				++num;
		}
		curr = center+ICoord(1,-1);
		if (pixelInBounds(curr, size) == true){
			if (list[curr] == type)
				++num;
		}
		curr = center+ICoord(-1,1);
		if (pixelInBounds(curr, size) == true){
			if (list[curr] == type)
				++num;
		}
	}
	
	return num;
}

bool FixBorders::pixelInBounds(ICoord pxl, ICoord size) {
  int xx = (pxl)(0);
  int yy = (pxl)(1);
  if ( (xx<0) || (xx>=size(0)) || (yy<0) || (yy>=size(1)) )
    return false;
  return true;
}


/*
CLear all regions of the same color that contains less than threshold pixels. 
*/
DoubleArray FixBorders::deleteSmallClumps(DoubleArray gray, int threshold){
	ICoord center;
	ICoord size = gray.size();
	int num;
	int num1;
	DoubleArray classified = DoubleArray(size);
	for (DoubleArray::iterator i = gray.begin();i !=gray.end(); ++i){
		center = i.coord();
		num1 = numSideEmpty(center, gray, 0, 0);
		if (gray[center] == 1){
			classified = DoubleArray(size);
			classified[center] = 1;
			num = numPixelsInArea(center, gray[center], gray, classified, threshold);
			++num;
			if (num < threshold){
				for (DoubleArray::iterator i = gray.begin();i !=gray.end(); ++i){
					center = i.coord();
					if (classified[center] == 1){
						gray[center] = (int)0;
					}
				}
			}
		}
	}
	return gray;
}

/*
Connects edges that are within 2 pixels of each other. 
*/
DoubleArray FixBorders::connectBorders(DoubleArray gray){
	ICoord center;
	ICoord size = gray.size();
	for (DoubleArray::iterator i = gray.begin();i !=gray.end(); ++i){
		center = i.coord();
		if (gray[center] == 1){
			if ((pixelInBounds(center + ICoord(1,-1), size) == true) && (gray[center + ICoord(1,-1)] == 0)){
				if ((pixelInBounds(center + ICoord(2,-2), size) == true) && (gray[center + ICoord(2,-2)] == 1)){
					gray[center + ICoord(1,-1)] = 1;
				}
			}
			if ((pixelInBounds(center + ICoord(-1,-1), size) == true) && (gray[center + ICoord(-1,-1)] == 0)){
				if ((pixelInBounds(center + ICoord(-2,-2), size) == true) && (gray[center + ICoord(-2,-2)] == 1)){
					gray[center + ICoord(-1,-1)] = 1;
				}
			}
			if ((pixelInBounds(center + ICoord(1,1), size) == true) && (gray[center + ICoord(1,1)] == 0)){
				if ((pixelInBounds(center + ICoord(2,2), size) == true) && (gray[center + ICoord(2,2)] == 1)){
					gray[center + ICoord(1,1)] = 1;
				}
			}
			if ((pixelInBounds(center + ICoord(-1,1), size) == true) && (gray[center + ICoord(-1,1)] == 0)){
				if ((pixelInBounds(center + ICoord(-2,2), size) == true) && (gray[center + ICoord(-2,2)] == 1)){
					gray[center + ICoord(-1,1)] = 1;
				}
			}
			if ((pixelInBounds(center + ICoord(1,0), size) == true) && (gray[center + ICoord(1,0)] == 0)){
				if ((pixelInBounds(center + ICoord(2,0), size) == true) && (gray[center + ICoord(2,0)] == 1)){
					gray[center + ICoord(1,0)] = 1;
				}
			}
			if ((pixelInBounds(center + ICoord(-1,0), size) == true) && (gray[center + ICoord(-1,0)] == 0)){
				if ((pixelInBounds(center + ICoord(-2,0), size) == true) && (gray[center + ICoord(-2,0)] == 1)){
					gray[center + ICoord(-1,0)] = 1;
				}
			}
			if ((pixelInBounds(center + ICoord(0,1), size) == true) && (gray[center + ICoord(0,1)] == 0)){
				if ((pixelInBounds(center + ICoord(0,2), size) == true) && (gray[center + ICoord(0,2)] == 1)){
					gray[center + ICoord(0,1)] = 1;
				}
			}
			if ((pixelInBounds(center + ICoord(0,-1), size) == true) && (gray[center + ICoord(0,-1)] == 0)){
				if ((pixelInBounds(center + ICoord(0,-2), size) == true) && (gray[center + ICoord(0,-2)] == 1)){
					gray[center + ICoord(0,-1)] = 1;
				}
			}
		}
	}
	return gray;
}


/*

Returns number of pixels in area if it is less than threshold. If it isnt then returns threshold (to avoid overflow for large regions).
*/
int FixBorders::numPixelsInArea(ICoord coord, double curr, DoubleArray changed, DoubleArray &classified, double threshold){
		/* below */
	ICoord size = changed.size();
	ICoord r = coord + ICoord (0,1);
	int num = 0;
	if ((pixelInBounds(r, size) == true) && (classified[r] == 0) && (changed[r] == curr)) {
		classified[r] = 1;
		++num;
		if (num >= threshold)
			return num;
		else
			num+=numPixelsInArea(r, curr, changed, classified, threshold);
	}
	/* above */
	r = coord + ICoord (0,-1);
	if ((pixelInBounds(r, size) == true) && (classified[r] == 0) && (changed[r] == curr)) {
		classified[r] = 1;
		++num;
		if (num >= threshold)
			return num;
		else
			num+=numPixelsInArea(r, curr, changed, classified,  threshold);
	}
	/*left */
	r = coord + ICoord (1,0);
	if ((pixelInBounds(r, size) == true) && (classified[r] == 0) && (changed[r] == curr)) {
		classified[r] = 1;
		++num;
		if (num >= threshold)
			return num;
		else
			num+=numPixelsInArea(r, curr, changed, classified, threshold);
	}
	/* right */
	r = coord + ICoord (-1,0);
	if ((pixelInBounds(r, size) == true) && (classified[r] == 0) && (changed[r] == curr)) {
		classified[r] = 1;
		++num;
		if (num >= threshold)
			return num;
		else
			num+=numPixelsInArea(r, curr, changed, classified,  threshold);
	}
	r = coord + ICoord (1,-1);
	if ((pixelInBounds(r, size) == true) && (classified[r] == 0) && (changed[r] == curr)) {
		classified[r] = 1;
		++num;
		if (num >= threshold)
			return num;
		else
			num+=numPixelsInArea(r, curr, changed, classified,  threshold);
	}
	
	r = coord + ICoord (-1,-1);
	if ((pixelInBounds(r, size) == true) && (classified[r] == 0) && (changed[r] == curr)) {
		classified[r] = 1;
		++num;
		if (num >= threshold)
			return num;
		else
			num+=numPixelsInArea(r, curr, changed, classified,  threshold);
	}
	
	r = coord + ICoord (-1,1);
	if ((pixelInBounds(r, size) == true) && (classified[r] == 0) && (changed[r] == curr)) {
		classified[r] = 1;
		++num;
		if (num >= threshold)
			return num;
		else
			num+=numPixelsInArea(r, curr, changed, classified, threshold);
	}
	
	r = coord + ICoord (1,1);
	if ((pixelInBounds(r, size) == true) && (classified[r] == 0) && (changed[r] == curr)) {
		classified[r] = 1;
		++num;
		if (num >= threshold)
			return num;
		else
			num+=numPixelsInArea(r, curr, changed, classified,  threshold);
	}

	return num;
}


